
subroutine CalcEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom, AtomType)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: AtomType
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8), dimension(Dim)  :: rij, Posi
    real(8) :: d2, id2, id6, id12
    real(8) :: rc2, Shift
    real(8) :: halfk
    integer :: i, j
    halfk = k/2.
    PEnergy = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            ! compute particle-particle separation vector
            rij = Pos(j,:) - Posi
            ! compute rij as minimum image distance
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij * rij)
            if (AtomType(i) == 1 .and. j == i + 1 .and. mod(j, M) > 0) then
                ! atoms are polymer atoms directly bonded to eachother, harmonic potential
                PEnergy = PEnergy + halfk*(sqrt(d2) - r0)**(2)
            else
                ! atoms are not bonded, lennard-jones potential
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2
                id6 = id2 * id2 * id2
                id12 = id6 * id6
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift
            endif
        enddo
    enddo




end subroutine


subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom, AtomType)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: AtomType
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    !======== YOUR CODE HERE ========
    real(8), dimension(Dim)  :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc2, Shift
    real(8) :: halfk
    integer :: i, j
    halfk = k/2.
    PEnergy = 0.
    Forces = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    rc2 = rc * rc
    do i = 0, NAtom - 1
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            ! compute particle-particle separation vector
            rij = Pos(j,:) - Posi
            ! compute rij as minimum image distance
            rij = rij - L*dnint(rij/L)
            d2 = sum(rij * rij)
            if (AtomType(i) == 1 .and. j == i + 1 .and. mod(j, M) > 0) then
                ! atoms are polymer atoms directly bonded to eachother, harmonic potential
                d = sqrt(d2)
                PEnergy = PEnergy + halfk*(d - r0)**(2.)
                Fij = rij * k* (d-r0)/d
            else
                ! atoms are not bonded, lennard-jones potential
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2
                id6 = id2 * id2 * id2
                id12 = id6 * id6
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
            endif
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
        enddo
    enddo


end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom, AtomType)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    integer, intent(in), dimension(0:NAtom-1) :: AtomType
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom, AtomType)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine